/*sE12.h
low level functions for controlling the bus
*/

#define DATA_OFF     0
#define RDWR_OFF     1
#define BIT_NINE_OFF 1
#define IRQ_OFF      2
#define DTACK_OFFSET 2
#define ISOOUT_OFF   3
#define ISOIN_OFF    3
#define EDGE_CONTROL RDWR_OFF

#define IRQ_BIT  0x80
#define NINE_BIT 0x80

#define PLDR1_IDENT 0x72

#define RISING_EDGE 0x80
#define FALLING_EDGE 0

#define PLDWR(device,offset,data){outb(data,(device->address+offset));}

#define DTACK_ON(device){PLDWR(device,DTACK_OFFSET,1);wmb();}
#define DTACK_OFF(device){PLDWR(device,DTACK_OFFSET,0);wmb();}
#define READ_MODE(device){PLDWR(device,RDWR_OFF,(device->IRQedge|1));wmb();}
#define WRITE_MODE(device){PLDWR(device,RDWR_OFF,device->IRQedge);wmb();}
#define BUSOUT(device,data){PLDWR(device,DATA_OFF,data);wmb();}
#define ISO_OUT(device,data){PLDWR(device,ISOOUT_OFF,data);wmb();}
#define IRQ_EDGE_INIT(device) READ_MODE(device)

#define PLDRD(device,offset) inb(device->address+offset)

#define MIRQ_HIGH(device) (PLDRD(device,IRQ_OFF)&IRQ_BIT)
#define MIRQ_LOW(device) (!MIRQ_HIGH(device))

#define BIT_NINE_READ(device,data){data=PLDRD(device,BIT_NINE_OFF)&NINE_BIT;rmb();} 
#define DATA_PORT(device,data){data=PLDRD(device,DATA_OFF);rmb();}
#define ISO_IN(device,data){data=PLDRD(device,ISOIN_OFF);rmb();}



#define WAIT_FOR_IRQ_LOW(device){rmb();while(MIRQ_HIGH(device))rmb();}
#define WAIT_FOR_IRQ_HIGH(device){rmb();while(MIRQ_LOW(device))rmb();}


#define IDENT_CHECK(device,ident){ident=(PLDRD(device,BIT_NINE_OFF)&(~NINE_BIT));rmb();}

